@@ -5,7 +5,7 @@
  * pgpool: a language independent connection pool server for PostgreSQL
  * written by Tatsuo Ishii
  *
- * Copyright (c) 2003-2025	PgPool Global Development Group
+ * Copyright (c) 2003-2024	PgPool Global Development Group
  *
  * Permission to use, copy, modify, and distribute this software and
  * its documentation for any purpose and without fee is hereby
@@ -84,7 +84,7 @@ static bool connect_using_existing_connection(POOL_CONNECTION * frontend,
 static void check_restart_request(void);
 static void enable_authentication_timeout(void);
 static void disable_authentication_timeout(void);
-static int	wait_for_new_connections(int *fds, struct timeval *timeout, SockAddr *saddr);
+static int	wait_for_new_connections(int *fds, SockAddr *saddr);
 static void check_config_reload(void);
 static void get_backends_status(unsigned int *valid_backends, unsigned int *down_backends);
 static void validate_backend_connectivity(int front_end_fd);
@@ -137,11 +137,6 @@ struct timeval startTime;
 bool		stop_now = false;
 #endif
 
-/*
- * If true, connection pool has been initialized.
- */
-static bool connection_pool_initialized = false;
-
 /*
 * child main loop
 */
@@ -152,9 +147,6 @@ do_child(int *fds)
 	POOL_CONNECTION_POOL *volatile backend = NULL;
 	struct timeval now;
 	struct timezone tz;
-	struct timeval timeout;
-	static int	connected = 0;	/* non 0 if has been accepted connections from
-								 * frontend */
 
 	/* counter for child_max_connections.  "volatile" declaration is necessary
 	 * so that this is counted up even if long jump is issued due to
@@ -228,7 +220,6 @@ do_child(int *fds)
 	{
 		child_exit(POOL_EXIT_AND_RESTART);
 	}
-	connection_pool_initialized = true;
 
 	/*
 	 * Open pool_passwd in child process.  This is necessary to avoid the file
@@ -290,6 +281,8 @@ do_child(int *fds)
 			if (pool_config->child_max_connections > 0)
 				connections_count++;
 
+			pool_get_my_process_info()->client_connection_count++;
+
 			/* check if maximum connections count for this child reached */
 			if ((pool_config->child_max_connections > 0) &&
 				(connections_count >= pool_config->child_max_connections))
@@ -313,9 +306,6 @@ do_child(int *fds)
 	/* We can now handle ereport(ERROR) */
 	PG_exception_stack = &local_sigjmp_buf;
 
-	timeout.tv_sec = pool_config->child_life_time;
-	timeout.tv_usec = 0;
-
 	for (;;)
 	{
 		StartupPacket *sp;
@@ -337,10 +327,11 @@ do_child(int *fds)
 		/* Destroy session context for just in case... */
 		pool_session_context_destroy();
 
-		front_end_fd = wait_for_new_connections(fds, &timeout, &saddr);
+		front_end_fd = wait_for_new_connections(fds, &saddr);
+		pool_get_my_process_info()->wait_for_connect = 0;
 		if (front_end_fd == OPERATION_TIMEOUT)
 		{
-			if (pool_config->child_life_time > 0 && connected)
+			if (pool_config->child_life_time > 0 && pool_get_my_process_info()->connected)
 			{
 				ereport(DEBUG1,
 						(errmsg("child life %d seconds expired", pool_config->child_life_time)));
@@ -411,7 +402,7 @@ do_child(int *fds)
 			child_frontend = NULL;
 			continue;
 		}
-		connected = 1;
+		pool_get_my_process_info()->connected = 1;
 
 		/*
 		 * show ps status
@@ -420,6 +411,7 @@ do_child(int *fds)
 		snprintf(psbuf, sizeof(psbuf), "%s %s %s idle",
 				 sp->user, sp->database, remote_ps_data);
 		set_ps_display(psbuf, false);
+		set_process_status(IDLE);
 
 		/*
 		 * Initialize per session context
@@ -471,13 +463,12 @@ do_child(int *fds)
 		if (pool_config->log_disconnections)
 			log_disconnections(sp->database, sp->user);
 
-		timeout.tv_sec = pool_config->child_life_time;
-		timeout.tv_usec = 0;
-
 		/* increment queries counter if necessary */
 		if (pool_config->child_max_connections > 0)
 			connections_count++;
 
+		pool_get_my_process_info()->client_connection_count++;
+
 		/* check if maximum connections count for this child reached */
 		if ((pool_config->child_max_connections > 0) &&
 			(connections_count >= pool_config->child_max_connections))
@@ -511,7 +502,7 @@ backend_cleanup(POOL_CONNECTION * volatile *frontend, POOL_CONNECTION_POOL * vol
 	sp = MAIN_CONNECTION(backend)->sp;
 
 	/*
-	 * cache connection if connection cache configuration parameter is enabled
+	 * cach connection if connection cache configuration parameter is enabled
 	 * and frontend connection is not invalid
 	 */
 	if (sp && pool_config->connection_cache != 0 && frontend_invalid == false)
@@ -754,11 +745,8 @@ read_startup_packet(POOL_CONNECTION * cp)
 	}
 
 	/* The database defaults to their user name. */
-	if (sp->database == NULL)
-		sp->database = pstrdup(sp->user);
-	else if (sp->database[0] == '\0')
+	if (sp->database == NULL || sp->database[0] == '\0')
 	{
-		pfree(sp->database);
 		sp->database = pstrdup(sp->user);
 	}
 
@@ -1037,8 +1025,9 @@ static POOL_CONNECTION_POOL * connect_backend(StartupPacket *sp, POOL_CONNECTION
 
 		for (i = 0; i < NUM_BACKENDS; i++)
 		{
-			if (VALID_BACKEND(i) && CONNECTION_SLOT(backend, i))
+			if (VALID_BACKEND(i))
 			{
+
 				/* set DB node id */
 				pool_set_db_node_id(CONNECTION(backend, i), i);
 
@@ -1090,7 +1079,7 @@ static POOL_CONNECTION_POOL * connect_backend(StartupPacket *sp, POOL_CONNECTION
 }
 
 /*
- * signal handler for SIGTERM, SIGINT and SIGQUUT
+ * signal handler for SIGTERM, SIGINT and SIGQUIT
  */
 static RETSIGTYPE die(int sig)
 {
@@ -1160,43 +1149,51 @@ static RETSIGTYPE close_idle_connection(int sig)
 	POOL_CONNECTION_POOL *p = pool_connection_pool;
 	ConnectionInfo *info;
 	int			save_errno = errno;
-	int			main_node_id;
 
 	/*
-	 * DROP DATABSE is ongoing.
+	 * DROP DATABASE is ongoing.
 	 */
 	if (ignore_sigusr1)
 		return;
 
+#ifdef NOT_USED
+	ereport(DEBUG1,
+			(errmsg("close connection request received")));
+#endif
+
 	for (j = 0; j < pool_config->max_pool; j++, p++)
 	{
-		main_node_id = in_use_backend_id(p);
-		if (main_node_id < 0)
+		if (!MAIN_CONNECTION(p))
+			continue;
+		if (!MAIN_CONNECTION(p)->sp)
+			continue;
+		if (MAIN_CONNECTION(p)->sp->user == NULL)
 			continue;
 
-		if (!CONNECTION_SLOT(p, main_node_id))
-			continue;
-		if (!CONNECTION_SLOT(p, main_node_id)->sp)
-			continue;
-		if (CONNECTION_SLOT(p, main_node_id)->sp->user == NULL)
-			continue;
-
-		if (CONNECTION_SLOT(p, main_node_id)->closetime > 0)	/* idle connection? */
+		if (MAIN_CONNECTION(p)->closetime > 0)	/* idle connection? */
 		{
-			bool	freed = false;
+#ifdef NOT_USED
+			ereport(DEBUG1,
+					(errmsg("closing idle connection"),
+					 errdetail("user: %s database: %s", MAIN_CONNECTION(p)->sp->user, MAIN_CONNECTION(p)->sp->database)));
+#endif
 
 			pool_send_frontend_exits(p);
 
 			for (i = 0; i < NUM_BACKENDS; i++)
 			{
-				if (!CONNECTION_SLOT(p, i))
+				if (!VALID_BACKEND(i))
 					continue;
 
-				if (!freed)
+				if (i == 0)
 				{
+					/*
+					 * only first backend allocated the memory for the start
+					 * up packet
+					 */
 					pool_free_startup_packet(CONNECTION_SLOT(p, i)->sp);
 					CONNECTION_SLOT(p, i)->sp = NULL;
-					freed = true;
+
 				}
 				pool_close(CONNECTION(p, i));
 			}
@@ -1305,7 +1302,7 @@ child_will_go_down(int code, Datum arg)
 	}
 
 	/* let backend know now we are exiting */
-	if (connection_pool_initialized)
+	if (pool_connection_pool)
 		close_all_backend_connections();
 }
 void
@@ -1440,7 +1437,7 @@ static void
 check_restart_request(void)
 {
 	/*
-	 * Check if restart request is set because of failback event happend.  If
+	 * Check if restart request is set because of failback event happened.  If
 	 * so, exit myself with exit code 1 to be restarted by pgpool parent.
 	 */
 	if (pool_get_my_process_info()->need_to_restart)
@@ -1457,12 +1454,12 @@ check_restart_request(void)
 /*
  * wait_for_new_connections()
  * functions calls select on sockets and wait for new client
- * to connect, on successfull connection returns the socket descriptor
- * and returns -1 if timeout has occured
+ * to connect, on successful connection returns the socket descriptor
+ * and returns -1 if timeout has occurred
  */
 
 static int
-wait_for_new_connections(int *fds, struct timeval *timeout, SockAddr *saddr)
+wait_for_new_connections(int *fds, SockAddr *saddr)
 {
 	fd_set		rmask;
 	int			numfds;
@@ -1480,10 +1477,8 @@ wait_for_new_connections(int *fds, struct timeval *timeout, SockAddr *saddr)
 	static int	cnt;
 #endif
 
-	struct timeval *timeoutval;
-	struct timeval tv1,
-				tv2,
-				tmback = {0, 0};
+	struct timeval *timeout;
+	struct timeval timeoutdata;
 
 	for (walk = fds; *walk != -1; walk++)
 		socket_set_nonblock(*walk);
@@ -1493,24 +1488,7 @@ wait_for_new_connections(int *fds, struct timeval *timeout, SockAddr *saddr)
 	else
 		set_ps_display("wait for connection request", false);
 
-	memcpy((char *) &rmask, (char *) &readmask, sizeof(fd_set));
-
-	if (timeout->tv_sec == 0 && timeout->tv_usec == 0)
-		timeoutval = NULL;
-	else
-	{
-		timeoutval = timeout;
-		tmback.tv_sec = timeout->tv_sec;
-		tmback.tv_usec = timeout->tv_usec;
-		gettimeofday(&tv1, NULL);
-
-#ifdef DEBUG
-		ereport(DEBUG3,
-				(errmsg("before select = {%d, %d}", timeoutval->tv_sec, timeoutval->tv_usec)));
-		ereport(DEBUG3,
-				(errmsg("g:before select = {%d, %d}", tv1.tv_sec, tv1.tv_usec)));
-#endif
-	}
+	set_process_status(WAIT_FOR_CONNECT);
 
 	/*
 	 * If child life time is disabled and serialize_accept is on, we serialize
@@ -1567,7 +1545,41 @@ wait_for_new_connections(int *fds, struct timeval *timeout, SockAddr *saddr)
 				(errmsg("LOCKING select()")));
 	}
 
-	numfds = select(nsocks, &rmask, NULL, NULL, timeoutval);
+	for (;;)
+	{
+		/* check backend timer is expired */
+		if (backend_timer_expired)
+		{
+			pool_backend_timer();
+			backend_timer_expired = 0;
+		}
+
+		/* prepare select */
+		memcpy((char *) &rmask, (char *) &readmask, sizeof(fd_set));
+		if (pool_config->child_life_time > 0)
+		{
+			timeoutdata.tv_sec = 1;
+			timeoutdata.tv_usec = 0;
+			timeout = &timeoutdata;
+		}
+		else
+		{
+			timeout = NULL;
+		}
+
+		numfds = select(nsocks, &rmask, NULL, NULL, timeout);
+
+		/* not timeout*/
+		if (numfds != 0)
+			break;
+
+		/* timeout */
+		pool_get_my_process_info()->wait_for_connect++;
+
+		if (pool_get_my_process_info()->wait_for_connect > pool_config->child_life_time)
+			return OPERATION_TIMEOUT;
+
+	}
 
 	save_errno = errno;
 
@@ -1585,40 +1597,6 @@ wait_for_new_connections(int *fds, struct timeval *timeout, SockAddr *saddr)
 		backend_timer_expired = 0;
 	}
 
-	/*
-	 * following code fragment computes remaining timeout val in a portable
-	 * way. Linux does this automatically but other platforms do not.
-	 */
-	if (timeoutval)
-	{
-		gettimeofday(&tv2, NULL);
-
-		tmback.tv_usec -= tv2.tv_usec - tv1.tv_usec;
-		tmback.tv_sec -= tv2.tv_sec - tv1.tv_sec;
-
-		if (tmback.tv_usec < 0)
-		{
-			tmback.tv_sec--;
-			if (tmback.tv_sec < 0)
-			{
-				timeout->tv_sec = 0;
-				timeout->tv_usec = 0;
-			}
-			else
-			{
-				tmback.tv_usec += 1000000;
-				timeout->tv_sec = tmback.tv_sec;
-				timeout->tv_usec = tmback.tv_usec;
-			}
-		}
-#ifdef DEBUG
-		ereport(DEBUG3,
-				(errmsg("g:after select = {%d, %d}", tv2.tv_sec, tv2.tv_usec)));
-		ereport(DEBUG3,
-				(errmsg("after select = {%d, %d}", timeout->tv_sec, timeout->tv_usec)));
-#endif
-	}
-
 	errno = save_errno;
 
 	if (numfds == -1)
@@ -1630,12 +1608,6 @@ wait_for_new_connections(int *fds, struct timeval *timeout, SockAddr *saddr)
 				 errdetail("select on socket failed with error : \"%m\"")));
 	}
 
-	/* timeout */
-	if (numfds == 0)
-	{
-		return OPERATION_TIMEOUT;
-	}
-
 	for (walk = fds; *walk != -1; walk++)
 	{
 		if (FD_ISSET(*walk, &rmask))
@@ -1739,10 +1711,9 @@ check_config_reload(void)
 		if (pool_config->enable_pool_hba)
 		{
 			load_hba(get_hba_file_name());
+			if (strcmp("", pool_config->pool_passwd))
+				pool_reopen_passwd_file();
 		}
-		if (strcmp("", pool_config->pool_passwd))
-			pool_reopen_passwd_file();
-
 		got_sighup = 0;
 	}
 }
@@ -2137,3 +2108,9 @@ static int opt_sort(const void *a, const void *b)
 {
 	return strcmp( *(char **)a, *(char **)b);
 }
+
+void
+set_process_status(ProcessStatus status)
+{
+	pool_get_my_process_info()->status = status;
+}
