@@ -143,9 +143,6 @@ do_worker_child(void)
 
 	MemoryContextSwitchTo(TopMemoryContext);
 
-	/* Initialize my backend status */
-	pool_initialize_private_backend_status();
-
 	/* Initialize per process context */
 	pool_init_process_context();
 
@@ -163,9 +160,15 @@ do_worker_child(void)
 
 	for (;;)
 	{
+		WD_STATES	wd_status;
+
 		MemoryContextSwitchTo(WorkerMemoryContext);
 		MemoryContextResetAndDeleteChildren(WorkerMemoryContext);
-		WD_STATES	wd_status;
+		/*
+		 * Since WorkerMemoryContext is used for "slots", we need to clear it
+		 * so that new slots are allocated later on.
+		 */
+		memset(slots, 0, sizeof(slots));
 
 		CHECK_REQUEST;
 
@@ -351,6 +354,20 @@ check_replication_time_lag(void)
 		return;
 	}
 
+	if (!VALID_BACKEND(REAL_PRIMARY_NODE_ID))
+	{
+		/*
+		 * No need to check replication delay if primary is down. This could
+		 * happen if ALWAYS_PRIMARY flag is on because REAL_PRIMARY_NODE_ID
+		 * macro returns the node id which ALWAYS_PRIMARY flag is set to.  If
+		 * we do not check this, subsequent test (i == PRIMARY_NODE_ID) in the
+		 * for loop below will return unexpected result because
+		 * PRIMARY_NODE_ID macro returns MAIN_NODE_ID, which could be a
+		 * standby server.
+		 */
+		return;
+	}
+
 	/*
 	 * Register a error context callback to throw proper context message
 	 */
@@ -379,7 +396,7 @@ check_replication_time_lag(void)
 
 		if (server_version[i] == 0)
 		{
-			query = "SELECT current_setting('server_version_num')";
+			query = "SELECT pg_catalog.current_setting('server_version_num')";
 
 			/*
 			 * Get backend server version. If the query fails, keep previous
@@ -397,21 +414,21 @@ check_replication_time_lag(void)
 		if (PRIMARY_NODE_ID == i)
 		{
 			if (server_version[i] >= PG10_SERVER_VERSION)
-				query = "SELECT pg_current_wal_lsn()";
+				query = "SELECT pg_catalog.pg_current_wal_lsn()";
 			else
-				query = "SELECT pg_current_xlog_location()";
+				query = "SELECT pg_catalog.pg_current_xlog_location()";
 
 			if (server_version[i] == PG91_SERVER_VERSION)
-				stat_rep_query = "SELECT application_name, state, '' AS sync_state FROM pg_stat_replication";
+				stat_rep_query = "SELECT application_name, state, '' AS sync_state FROM pg_catalog.pg_stat_replication";
 			else if (server_version[i] > PG91_SERVER_VERSION)
-				stat_rep_query = "SELECT application_name, state, sync_state FROM pg_stat_replication";
+				stat_rep_query = "SELECT application_name, state, sync_state FROM pg_catalog.pg_stat_replication";
 		}
 		else
 		{
 			if (server_version[i] >= PG10_SERVER_VERSION)
-				query = "SELECT pg_last_wal_replay_lsn()";
+				query = "SELECT pg_catalog.pg_last_wal_replay_lsn()";
 			else
-				query = "SELECT pg_last_xlog_replay_location()";
+				query = "SELECT pg_catalog.pg_last_xlog_replay_location()";
 
 			active_standby_node++;
 		}
