@@ -559,11 +559,11 @@ is_system_catalog(char *table_name)
 /*
  * Query to know if the target table belongs pg_catalog schema.
  */
-#define ISBELONGTOPGCATALOGQUERY "SELECT count(*) FROM pg_catalog.pg_class AS c, pg_namespace AS n WHERE c.relname = '%s' AND c.relnamespace = n.oid AND n.nspname = 'pg_catalog'"
+#define ISBELONGTOPGCATALOGQUERY "SELECT count(*) FROM pg_catalog.pg_class AS c, pg_catalog.pg_namespace AS n WHERE c.relname = '%s' AND c.relnamespace = n.oid AND n.nspname = 'pg_catalog'"
 
-#define ISBELONGTOPGCATALOGQUERY2 "SELECT count(*) FROM pg_catalog.pg_class AS c, pg_namespace AS n WHERE c.oid = pgpool_regclass('\"%s\"') AND c.relnamespace = n.oid AND n.nspname = 'pg_catalog'"
+#define ISBELONGTOPGCATALOGQUERY2 "SELECT count(*) FROM pg_catalog.pg_class AS c, pg_catalog.pg_namespace AS n WHERE c.oid = pgpool_regclass('\"%s\"') AND c.relnamespace = n.oid AND n.nspname = 'pg_catalog'"
 
-#define ISBELONGTOPGCATALOGQUERY3 "SELECT count(*) FROM pg_catalog.pg_class AS c, pg_namespace AS n WHERE c.oid = pg_catalog.to_regclass('\"%s\"') AND c.relnamespace = n.oid AND n.nspname = 'pg_catalog'"
+#define ISBELONGTOPGCATALOGQUERY3 "SELECT count(*) FROM pg_catalog.pg_class AS c, pg_catalog.pg_namespace AS n WHERE c.oid = pg_catalog.to_regclass('\"%s\"') AND c.relnamespace = n.oid AND n.nspname = 'pg_catalog'"
 
 	bool		result;
 	static POOL_RELCACHE * relcache;
@@ -650,7 +650,7 @@ is_temp_table(char *table_name)
  * regclass (or its variant) here, because temporary tables never have
  * schema qualified name.
  */
-#define ISTEMPQUERY83 "SELECT count(*) FROM pg_catalog.pg_class AS c, pg_namespace AS n WHERE c.relname = '%s' AND c.relnamespace = n.oid AND n.nspname ~ '^pg_temp_'"
+#define ISTEMPQUERY83 "SELECT count(*) FROM pg_catalog.pg_class AS c, pg_catalog.pg_namespace AS n WHERE c.relname = '%s' AND c.relnamespace = n.oid AND n.nspname ~ '^pg_temp_'"
 
 /*
  * Query to know if the target table is a temporary one.  This query
@@ -1314,7 +1314,7 @@ select_table_walker(Node *node, void *context)
 			num_oids = ctx->num_oids++;
 
 			ctx->table_oids[num_oids] = oid;
-			strlcpy(ctx->table_names[num_oids], table, POOL_NAMEDATALEN);
+			strlcpy(ctx->table_names[num_oids], table, NAMEDATALEN);
 
 			ereport(DEBUG1,
 					(errmsg("extracting table oids from SELECT statement"),
@@ -1377,11 +1377,11 @@ make_function_name_from_funccall(FuncCall *fcall)
 {
 	/*
 	 * Function name. Max size is calculated as follows: schema
-	 * name(POOL_NAMEDATALEN byte) + quotation marks for schema name(2 byte) +
-	 * period(1 byte) + table name (POOL_NAMEDATALEN byte) + quotation marks
+	 * name(NAMEDATALEN byte) + quotation marks for schema name(2 byte) +
+	 * period(1 byte) + table name (NAMEDATALEN byte) + quotation marks
 	 * for table name(2 byte) + NULL(1 byte)
 	 */
-	static char funcname[POOL_NAMEDATALEN * 2 + 1 + 2 * 2 + 1];
+	static char funcname[NAMEDATALEN * 2 + 1 + 2 * 2 + 1];
 	List 	   *names;
 
 	if(fcall == NULL)
@@ -1398,29 +1398,29 @@ make_function_name_from_funccall(FuncCall *fcall)
 	{
 		case 1:
 			strcat(funcname, "\"");
-			strncat(funcname, strVal(linitial(names)), POOL_NAMEDATALEN);
+			strncat(funcname, strVal(linitial(names)), NAMEDATALEN);
 			strcat(funcname, "\"");
 			break;
 		case 2:
 			strcat(funcname, "\"");
-			strncat(funcname, strVal(linitial(names)), POOL_NAMEDATALEN);
+			strncat(funcname, strVal(linitial(names)), NAMEDATALEN);
 			strcat(funcname, "\"");
 
 			strcat(funcname, ".");
 
 			strcat(funcname, "\"");
-			strncat(funcname, strVal(lsecond(names)), POOL_NAMEDATALEN);
+			strncat(funcname, strVal(lsecond(names)), NAMEDATALEN);
 			strcat(funcname, "\"");
 			break;
 		case 3:
 			strcat(funcname, "\"");
-			strncat(funcname, strVal(lsecond(names)), POOL_NAMEDATALEN);
+			strncat(funcname, strVal(lsecond(names)), NAMEDATALEN);
 			strcat(funcname, "\"");
 
 			strcat(funcname, ".");
 
 			strcat(funcname, "\"");
-			strncat(funcname, strVal(lthird(names)), POOL_NAMEDATALEN);
+			strncat(funcname, strVal(lthird(names)), NAMEDATALEN);
 			strcat(funcname, "\"");
 
 			break;
@@ -1446,11 +1446,11 @@ make_table_name_from_rangevar(RangeVar *rangevar)
 {
 	/*
 	 * Table name. Max size is calculated as follows: schema
-	 * name(POOL_NAMEDATALEN byte) + quotation marks for schema name(2 byte) +
-	 * period(1 byte) + table name (POOL_NAMEDATALEN byte) + quotation marks
+	 * name(NAMEDATALEN byte) + quotation marks for schema name(2 byte) +
+	 * period(1 byte) + table name (NAMEDATALEN byte) + quotation marks
 	 * for table name(2 byte) + NULL(1 byte)
 	 */
-	static char tablename[POOL_NAMEDATALEN * 2 + 1 + 2 * 2 + 1];
+	static char tablename[NAMEDATALEN * 2 + 1 + 2 * 2 + 1];
 
 	if (rangevar == NULL)
 	{
@@ -1472,7 +1472,7 @@ make_table_name_from_rangevar(RangeVar *rangevar)
 	if (rangevar->schemaname)
 	{
 		strcat(tablename, "\"");
-		strncat(tablename, rangevar->schemaname, POOL_NAMEDATALEN);
+		strncat(tablename, rangevar->schemaname, NAMEDATALEN);
 		strcat(tablename, "\"");
 		strcat(tablename, ".");
 	}
@@ -1486,7 +1486,7 @@ make_table_name_from_rangevar(RangeVar *rangevar)
 	}
 
 	strcat(tablename, "\"");
-	strncat(tablename, rangevar->relname, POOL_NAMEDATALEN);
+	strncat(tablename, rangevar->relname, NAMEDATALEN);
 	strcat(tablename, "\"");
 
 	ereport(DEBUG1,
